#!/bin/bash
#
# Functions used by testsuite.sh


function init_trace {
  process_template index_initial.tmpl "${REPORT_DIR}/index.html"
}

function trace {
  echo "$1"
  echo "${1}<br>" >> "${REPORT_DIR}/log.html"
}

function check_dir {
  # create report dir if needed
  if [ ! -d "$1" ]; then
    mkdir -p "$1"
    if [ $? -ne 0 ]; then
      exit 1
    fi
  fi
}

function process_template {
  local tmpl_file=${TEMPLATE_DIR}/$1
  local tmpl_str=$(< ${tmpl_file})
  local tmpl_result
  local dir

  dir="$(dirname ${tmpl_file})"
  [ -d  "${dir}" ] || mkdir -p "${dir}"
  dir="$(dirname ${2})"
  [ -d  "${dir}" ] || mkdir -p "${dir}"
  # here is the magic of the shell template ;-)
  eval "tmpl_result=\"${tmpl_str}\""
  echo "$tmpl_result" >> $2
}

function getPlaylistFile {
    echo "Use $playlistFile"
    playlist=$(< $playlistFile)
	OLDIFS="${IFS}"
	IFS=$'\n'
	local i=0
	
	for sample in ${playlist}; do
	  playlistMRLs[${i}]="${sample}"
	  playlistTitles[${i}]="${sample}"
	  i=$[${i} + 1]
	done
	IFS="${OLDIFS}"
}

# return the list of samples
function getDefaultPlaylist {
    echo "Use default playlist"
	OLDIFS="${IFS}"
	IFS=$'\n'
	playlist=$(${find} "${SAMPLE_DIR}" ${find_args} -type f | ${sort})
	local i=0
	
	for sample in ${playlist}; do
          # remove .config and .info files from the list
	  if [ "${sample%.config}" == "${sample}" ]; then
	    if [ "${sample%.info}" == "${sample}" ]; then
		  startPos=$[${#SAMPLE_DIR} + 1]
		  playlistMRLs[${i}]="${sample}"
		  playlistTitles[${i}]="${sample:${startPos}}"
		  i=$[${i} + 1]
            fi
	  fi
	done
	IFS="${OLDIFS}"
}

# load the config file associated to a sample
function loadSampleConfigFile {
    
	testBasename="$1"
	sampleMRL="$2"
	configFile="${sampleMRL}.config"
	local i=0
	
	if [ -f "${configFile}" ]; then
		{
			while read line; do
				# check test name
				param="${line#${testBasename}:}"
				if [ "${param}" != "${line}" ]; then
					test_args[${i}]="${param}"
					i=$[${i} + 1]
				fi
			done
		} < "${configFile}"
	fi
}

# return a sample info
function getSampleInfo {
        local sample="$1"
        local infoName="$2"
        local infoNameLen=${#infoName}

        local line=$(grep "${infoName}:" < "${RESULT_DIR}/`echo ${SAMPLE_DIR}/${sample}.info | sed -e 's,/,_,g'`")
        local info=${line:$(( ${infoNameLen} + 1 ))}
        echo ${info}
}

# replace '_' by ' ' in sample names
function insertSpaces {
        echo "$1" | sed 's/_/ /g'
}

# return the result of a test
function getTestResult {
        local run="$1"
        local testName="$2"
        local testNameLen=${#testName}

        local line=$(grep "${testName}:" < "${RUN_REPORT_DIR}/${run}/summary.txt")
        local info=${line:$(( ${testNameLen} + 1 ))}

        echo ${info}
}

# return the rate success of a test
function getTestRateSuccess {
        local run="$1"
        local testName="$2"
        local testNameLen=${#testName}

        local line=$(grep "${testName}:" < "${RUN_REPORT_DIR}/${run}/success_rate.txt")
        local info=${line:$(( ${testNameLen} + 1 ))}

        echo ${info}
}

# return the previous Run Name
function getLatestRun {
#        ls -1r "${RUN_REPORT_DIR}" | grep -A 1 "${ISODATE}" | tail -n 1
	ls -1r "${RUN_REPORT_DIR}" | grep '....-..-..' | ${sort} | tail -n 1
}

# return the previous Run Name
function getPreviousRun {
#        ls -1r "${RUN_REPORT_DIR}" | grep -A 2 "${ISODATE}" | tail -n 1
	ls -1r "${RUN_REPORT_DIR}" | grep '....-..-..' | ${sort} | tail -n 2 | head -n 1
}
