#!/bin/bash
#
# TestSuite for xine-lib
#

export BIN_DIR=$(dirname $0)
export TESTSUITE_DIR=${BIN_DIR}/..
export REPORTS_DIR=/var/www/html/testsuite/reports
export TEMPLATE_DIR=${TESTSUITE_DIR}/templates
export TEST_LIST=${TESTSUITE_DIR}/tests/test_full.list
[ -f ${BIN_DIR}/config.sh ] && . ${BIN_DIR}/config.sh
STARTDATE=$(${date})
ISODATE=$(${date} --iso-8601)
. ${BIN_DIR}/functions.sh

echo

echo "Configuration:"
echo "  REPORTS_DIR: ${REPORTS_DIR}"

testReportFile="${REPORTS_DIR}"/index.html
testReportFile2="${REPORTS_DIR}"/testsuite.inc
rm -f ${testReportFile}

process_template platforms/header.tmpl "${testReportFile}"
echo "  <TR class='TableHeader'>" >> "${testReportFile}"
echo "    <TH>Test</TH>" >> "${testReportFile}"
for platform in ${PLATFORMS};
do
  RUN_REPORT_DIR=${REPORTS_DIR}/${platform}
  LATEST_RUN=$(getLatestRun)
  platform_name=$(< "${TESTSUITE_DIR}"/results/${platform}/.platform)
  echo "    <TH><A HREF=\"${platform}/index.html\">${platform_name}</A><BR><A HREF=\"${platform}/${LATEST_RUN}/index.html\">${LATEST_RUN}</A></TH>" >> "${testReportFile}"
done
echo '  </TR>' >> "${testReportFile}"

for testBasename in $(< ${TEST_LIST});
do
  echo "${testBasename}"

  echo '  <TR>' >> "${testReportFile}"
  echo "    <TD>${testBasename}</TD>" >> "${testReportFile}"

  for platform in ${PLATFORMS}; do
    echo "  $platform"
    RESULT_DIR=${TESTSUITE_DIR}/results/${platform}
    testResultDir=${RESULT_DIR}/${testBasename}
    RUN_REPORT_DIR=${REPORTS_DIR}/${platform}
    LATEST_RUN=$(getLatestRun)
    PREVIOUS_RUN=$(getPreviousRun)
    PREVIOUS_RUN_RESULT=$(getTestResult "${PREVIOUS_RUN}" "${testBasename}")
    PREVIOUS_RUN_RATE_SUCCESS=$(getTestRateSuccess "${PREVIOUS_RUN}" "${testBasename}")
    echo "    LATEST_RUN: ${LATEST_RUN}"
    echo "    PREVIOUS_RUN: ${PREVIOUS_RUN}"
    echo "    PREVIOUS_RUN_RESULTS: ${PREVIOUS_RUN_RESULTS}"
    echo "    PREVIOUS_RUN_RATE_SUCCESS: ${PREVIOUS_RUN_RATE_SUCCESS}"

    SAMPLES_COUNT=0
    SAMPLES_OK=0
    SAMPLES_KO=0

    if [ -d ${testResultDir} ]; then
      for sampleResultDir in ${testResultDir}/sample*;
      do
        resultFile=${sampleResultDir}/result
        SAMPLES_COUNT=$[${SAMPLES_COUNT}+1]
        SAMPLE_STATUS=$(< "${resultFile}")
        if [ "${SAMPLE_STATUS}" -eq 0 ]; then
          SAMPLES_OK=$[${SAMPLES_OK}+1]
        else
          SAMPLES_KO=$[${SAMPLES_OK}+1]
        fi
      done
    fi
    if [ ${SAMPLES_COUNT} -ne 0 ]; then
      TEST_SUCCESS_PCT=$(echo "scale=2; 100 * ${SAMPLES_OK} / ${SAMPLES_COUNT}" | ${bc})
      TEST_SUCCESS_PCT_INT=$( echo "scale=0; print ${TEST_SUCCESS_PCT} / 1" | ${bc})
      echo "    TEST_SUCCESS_PCT: ${TEST_SUCCESS_PCT}"
      if [ ${SAMPLES_KO} -eq 0 ]; then
        if [ ! "${PREVIOUS_RUN_RESULT}" = "failure" ]; then
	  process_template platforms/ok_equal.tmpl "${testReportFile}"
        else
	  process_template platforms/ok_greater.tmpl "${testReportFile}"
        fi
      else
	if [ "${PREVIOUS_RUN_RATE_SUCCESS}" = "" ]; then
	    PREVIOUS_RUN_RATE_SUCCESS="${TEST_SUCCESS_PCT}"
	fi
	if [ $(echo "${PREVIOUS_RUN_RATE_SUCCESS} < ${TEST_SUCCESS_PCT}" | ${bc}) -eq 1 ]; then
	    process_template platforms/ko_greater.tmpl "${testReportFile}"
	elif [ $(echo "${PREVIOUS_RUN_RATE_SUCCESS} > ${TEST_SUCCESS_PCT}" | ${bc}) -eq 1 ]; then
	    process_template platforms/ko_lower.tmpl "${testReportFile}"
	else
	    process_template platforms/ko_equal.tmpl "${testReportFile}"
	fi
      fi
    else
      process_template platforms/noitem.tmpl "${testReportFile}"
      echo "    -"
    fi
  done
  echo '  </TR>' >> "${testReportFile}"
done
process_template platforms/footer.tmpl "${testReportFile}"
cat "${testReportFile}" | tail -n +24 | head -n -4 | sed -e 's,HREF=",HREF="testsuite/reports/,g' > "${testReportFile2}"
