#! /bin/bash

#
# fetch sources and build the module,
# see usage()
#

#set -o xtrace
#CVSROOT_LIB=:pserver:anonymous@xine.cvs.sourceforge.net:/cvsroot/xine
CVSROOT_LIB=:ext:valtri@xine.cvs.sourceforge.net:/cvsroot/xine
#CVSROOT_LIB=:local:${HOME}/CVS
CVSROOT_FF=:pserver:anonymous@mplayerhq.hu:/cvsroot/ffmpeg
CVSROOT_TOX=:pserver:anonymous@toxine.cvs.sourceforge.net:/cvsroot/toxine
#CVSROOT_TOX=:local:${HOME}/CVS

#AUTODIR=/home/valtri/xine-local/testsuite/testsuite/tests/build
AUTODIR=$(cd $(dirname $0); pwd)
CVS_ATTEMPTS=${CVS_ATTEMPTS:-5}
MAX_NUMBER=${MAX_NUMBER:-3}
#MINGW_READLINE_PREFIX=/mingw32/readline
MINGW_READLINE_PREFIX=/mingw32/GNUWin32
MINGW_ZLIB_PREFIX=/mingw32/zlib
UCLIBC_READLINE_PREFIX=/uclibc/readline


function usage() {
	cat << EOF
Usage: `basename $0`
	fetch module
	build MODULE [TARGET]
	build295 MODULE [TARGET]
	check MODULE [TARGET]
	package PROJECT VERSION TYPE
	set VARIABLE VALUE
	upload suite PROJECT1 [PROJECT2...]
	rem COMMENT

where MODULE is:
	xine-lib[-1.0]|xine-win32|pthreads-win32|zlib
	toxine|xiph|gwin32-readline|modplug

requires variables:
	$1 - module
	$2 - target
	PREFIX
	PROJECT
	BINSDIR
	TESTSUITE_(TARGET/DOWNLOAD/WORK/DIST)DIR
recommended variables:
	TIMESTAMP
EOF
	return 0
}


#
# make the download and build of the desired module
#
# requires:
#  MODULE
#  CVSROOT_*
#  AUTODIR
#  PREFIX
#
# recommends:
#   TIMESTAMP
#
function main {
#	echo "Module build script"
#	echo "  time: `${date} \"+%Y-%m-%d %H:%M:%S\"`"

	[ -d "${TESTSUITE_TARGETDIR}" ] || mkdir -p "${TESTSUITE_TARGETDIR}" || return $?
	[ -d "${TESTSUITE_WORKDIR}" ] || mkdir -p "${TESTSUITE_WORKDIR}" || return $?
	[ -d "${TESTSUITE_DISTDIR}" ] || mkdir -p "${TESTSUITE_DISTDIR}" || return $?
	[ -d "${TESTSUITE_DOWNLOADDIR}" ] || mkdir -p "${TESTSUITE_DOWNLOADDIR}" || return $?
	cd "${TESTSUITE_TARGETDIR}" || return $?

	case "$1" in
	fetch)
	MODULE="$2"
	modules_fetch || return $?
	;;

	build)
	MODULE="$2"
	TARGET="$3"
	echo "# module: '$MODULE', target: '$TARGET' (cross: '$CROSS')"

	# for distribution
	BUILDDIR="${TESTSUITE_WORKDIR}/build-${MODULE}"
	DESTDIR="${TESTSUITE_WORKDIR}/dest"
	echo -n "${MODULE} " >> "${TESTSUITE_WORKDIR}/all.list"
	modules_build || return $?
	echo -n "${MODULE} " >> "${TESTSUITE_WORKDIR}/success.list"
	;;

	build295)
	MODULE="$2"
	TARGET="$3"
	which gcc-2.95 >/dev/null 2>&1 && export CC=gcc-2.95
	which gcc295 >/dev/null 2>&1 && export CC=gcc295
	[ -z "${CC}" ] && echo "GCC 2.95 not found." && return -1
	BUILDDIR="${TESTSUITE_WORKDIR}/build-${MODULE}"
	DESTDIR="${TESTSUITE_WORKDIR}/dest295"
	CFLAGS_non295="${CFLAGS}"
	LDFLAGS_non295="${LDFLAGS}"
	CFLAGS=$(echo "${CFLAGS}" | sed -e 's/\-fpie//g')
	LDFLAGS=$(echo "${LDFLAGS}" | sed -e 's/\-pie//g')
	modules_build || return $?
	CFLAGS="${CFLAGS_non295}"
	LDFLAGS="${LDFLAGS_non295}"
	unset CC
	rm -Rf "${TESTSUITE_WORKDIR}/dest295" || return $?
	;;

	check)
	MODULE="$2"
#	TARGET="$3"
	BUILDDIR=${TESTSUITE_WORKDIR}/build-${MODULE}
	DESTDIR="${TESTSUITE_WORKDIR}/check"
	check_module || return $?
	;;

	package)
	[ -z "$4" ] && usage && return 1
	PROJECT="$2"
	VERSION="$3"
	TYPE="$4"
	echo "# package: '$PROJECT'-'$VERSION'.'$TYPE'"
	package_project "${PROJECT}" $(eval echo "${VERSION}") ${TYPE} || return $?
	;;

	set)
	[ -z "$3" ] && usage && return 1
	export $2="$3"
	;;

	upload)
	shift
	testsuite_postupload $@ || return $?
	;;

	rem)
	:
	;;

	*)
	usage
	return 1
	;;

	esac

	return 0
}


function modules_fetch() {
	echo "# modules_fetch: '$MODULE'"

	case "${MODULE}" in
	xine-lib)
	cvs_download "$CVSROOT_LIB" "" "${MODULE}" || return $?
	patch_common "${XINE_LIB_PATCHES}" "${XINE_LIB_FILES}" || return $?
	;;

	xine-lib-1.0)
	cvs_download "$CVSROOT_LIB" "xine-1_0" "xine-lib" || return $?
	patch_common "${XINE_LIB_PATCHES}" "${XINE_LIB_FILES}" || return $?
	;;

	xine-win32)
	cvs_download "$CVSROOT_LIB" "" "${MODULE}" || return $?
	patch_common "${XWIN32_PATCHES}" "${XWIN32_FILES}" || return $?
	;;

	toxine)
	cvs_download "$CVSROOT_TOX" "" "${MODULE}" || return $?
	patch_common "${TOXINE_PATCHES}" "${TOXINE_FILES}" || return $?
	;;

	xine-ui)
	cvs_download "$CVSROOT_LIB" "" "${MODULE}" || return $?
	patch_common "${XINE_UI_PATCHES}" "${XINE_UI_FILES}" || return $?
	;;

	*)
	echo "Unknown module ${MODULE}"
	return 1
	;;

	esac

	return 0
}


#
# build a module
#
function modules_build() {
	echo "# modules_build: '$MODULE', target: '$TARGET' (cross: '$CROSS')"
	echo "# prefix: '${PREFIX}', dest: '${DESTDIR}'"

	[ -d "${DESTDIR}${PREFIX}/bin" ] || mkdir -p ${DESTDIR}${PREFIX}/bin || return $?
	case "${MODULE}" in
		pthreads-win32)
		cp /mingw32/pthreads-win32/bin/pthreadGC2.dll ${DESTDIR}${PREFIX}/bin/ \
			|| return $?
		;;

		zlib)
		cp /mingw32/zlib/bin/zlib1.dll ${DESTDIR}${PREFIX}/bin/ || return $?
		;;

		xiph)
		cp /mingw32/xiph/bin/lib{ogg-0,vorbis-0,theora-0,speex-1}.dll \
			${DESTDIR}${PREFIX}/bin/ || return $?
		;;

		gwin32-readline)
		cp ${MINGW_READLINE_PREFIX}/bin/{history5,readline5,msvcp60}.dll \
			${DESTDIR}${PREFIX}/bin/ || return $?
		;;

		modplug)
		cp /mingw32/modplug/bin/libmodplug-0.dll ${DESTDIR}${PREFIX}/bin/ \
			|| return $?
		;;

		xine-lib)
		build_lib "${XINE_LIB_POST_PATCHES}" || return $?
		;;

		xine-lib-1.0)
		build_lib "${XINE_LIB_POST_PATCHES}" || return $?
		;;

		xine-win32)
		build_xwin32 || return $?
		;;

		toxine)
		# double build because of hardcoded plugin path
		build_toxine || return $?
		PREFIX="${TESTSUITE_DISTDIR}${PREFIX}" DESTDIR="" build_toxine || return $?
		;;

		xine-ui)
		build_xineui || return $?
		;;

		*)
		echo "Unknown module ${MODULE}"
		return 1
		;;

	esac

	return 0
}


#
# download xine sources
#
# requires:
#   $MODULE name of the module
#   $TIMESTAMP recommended
#   
function cvs_download() {
  local CVSROOT=${1};
  local CVSBRANCH=${2}
  local CVSMODULE=${3}

  local i
  local ARGS
  local TAG_ARG
  local TIMESTAMP_ARG

  local FETCHDIR="${TESTSUITE_TARGETDIR}/fetch"

  [ -z "$NO_DOWNLOAD" ] || return 0

  cd "${TESTSUITE_DOWNLOADDIR}"
  if [ -s "${MODULE}.tar.gz" ]; then
    echo "# ${MODULE} already downloaded"
    return 0
  fi
  if [ -f "${MODULE}.download_failed" ]; then
    echo "# ${MODULE} NOT downloaded"
    return 1
  fi

  if [ -n "${CVSBRANCH}" ]; then
    ARGS="-r ${CVSBRANCH} $ARGS"
  fi

  rm -Rf "${FETCHDIR}" || return $?
  mkdir -p "${FETCHDIR}" && cd "${FETCHDIR}" || return $?

  i=0
  while [ $i -lt ${CVS_ATTEMPTS} ]; do
    if [ -n "${TIMESTAMP}" ]; then
      cvs -q -z1 -d${CVSROOT} co -D "${TIMESTAMP}" ${ARGS} ${CVSMODULE}
    else
      cvs -q -z1 -d${CVSROOT} co ${ARGS} ${CVSMODULE}
    fi
    if [ x"$?" = x0 ]; then break; fi
    i=$(($i+1))
    sleep 20
  done
  if [ $i = ${CVS_ATTEMPTS} ]; then
    [ -d "${MODULE}" -a "${MODULE}" != "/" ] && rm -Rf "${MODULE}"
    touch "${MODULE}.download_failed"
    echo "  ${MODULE} download FAILED"
    return 1
  fi

  [ x"${TESTSUITE_DOWNLOADDIR}/${MODULE}" != x"/" ] && rm -Rf "${TESTSUITE_DOWNLOADDIR}/${MODULE}"
  mv "${CVSMODULE}" "${TESTSUITE_DOWNLOADDIR}/${MODULE}"
  rmdir "${FETCHDIR}" || return $?
  echo "  ${MODULE} download OK"

  cd "${TESTSUITE_DOWNLOADDIR}"
  tar -cf - "${MODULE}" | gzip > "${MODULE}.tar.gz"
  rm -Rf "${MODULE}"

  cd "${TESTSUITE_WORKDIR}"

  return 0
}


function patch_common() {
	local ret

	ret=0
	cd "${TESTSUITE_DOWNLOADDIR}"
	if [ ! -f "${MODULE}.patched" -o ! -d "${MODULE}" ]; then
		rm -Rf "${MODULE}" || return $?
		tar xzf "${MODULE}.tar.gz" || return $?
    		cd "${MODULE}" || return $?
		for patch in $1; do
			patch -p0 < "${AUTODIR}/${patch}" || ret=$?
		done
		for copy in $2; do
			file=${copy%%,*}
			dest=${copy##*,}
			cp "${AUTODIR}/${file}" "${dest}" || ret=$?
		done
		touch "../${MODULE}.patched" || return $?
	fi
	cd "${TESTSUITE_WORKDIR}"
	return $ret
}


#
# build xine-lib
# requires:
#	MODULE
#	AUTODIR
#	PREFIX
#	DESTDIR
#
function build_lib() {
	local ret
	local debug

	echo "Building ${MODULE} (target: ${TARGET}, cross: ${CROSS}, debug: ${DEBUG})"
	echo "DESTDIR: $DESTDIR, PREFIX: $PREFIX"

	debug=""; [ -z "${DEBUG}" ] || debug="-debug"
	[ -d "${BUILDDIR}" ] || mkdir -p "${BUILDDIR}" || return $?

	cd "${TESTSUITE_DOWNLOADDIR}/${MODULE}" || return $?
	[ -x ./configure ] || ./autogen.sh noconfig || return $?

	cd "${BUILDDIR}" || return $?

	echo "PATH='${PATH}'"
	echo "PKG_CONFIG_PATH='${PKG_CONFIG_PATH}'"
	echo "PKG_CONFIG_LIBDIR='${PKG_CONFIG_LIBDIR}'"
	echo "CFLAGS='${CFLAGS}'"
	echo "CPPFLAGS='${CPPFLAGS}'"
	echo "LDLAGS='${LDLAGS}'"
	echo "LIBS='${LIBS}'"
	echo "${TESTSUITE_DOWNLOADDIR}/${MODULE}/configure --prefix=${PREFIX} ${XINE_LIB_CONFIGURE_OPTS}"
	${TESTSUITE_DOWNLOADDIR}/${MODULE}/configure \
		--prefix=${PREFIX} ${XINE_LIB_CONFIGURE_OPTS} || return $?

	ret=0
	for patch in $1; do
		patch -p0 -f < "${AUTODIR}/${patch}" || ret=$?
	done

	${make} install${debug} DESTDIR=${DESTDIR} || return $?


	if [ x"${CROSS}" = x"i386-mingw32" -o x"${CROSS}" = x"i386-mingw32msvc" ]
	then
		rm -f "${DESTDIR}${PREFIX}/lib/xine/plugins/{*/*.a,*/*/*.a}"
		rm -f "${DESTDIR}${PREFIX}/lib/xine/plugins/*/xineplug_ao_out_directx.dll"
		if [ -z "NO_RUN_MSVC_LIB" ]; then
			(cd ${DESTDIR}${PREFIX}/lib &&\
			 cp ../bin/libxine-1.dll . &&\
			 /mingw32/VC60/LIB.EXE /machine:i386 /def:libxine-1.def; \
			 rm libxine-1.dll || return $?) || return $?
		fi

		[ -z "${DEBUG}" ] && ${CROSS}-strip ${DESTDIR}${PREFIX}/lib/xine/plugins/{*/,*/*/}*.dll
	fi

	cd ${TESTSUITE_WORKDIR}
	touch ${MODULE}.built || return $?

	return $ret
}


function build_xwin32() {
	local DEBUG_CFLAGS
	local XINE_LIB_PREFIX
	local XINE_CFLAGS
	local XINE_LIBS

      	XINE_LIB_PREFIX="${TESTSUITE_WORKDIR}/dest${PREFIX}"
	DEBUG_CFLAGS=""; [ -z "${DEBUG}" ] || DEBUG_CFLAGS="-g "

	cd "${TESTSUITE_WORKDIR}" || return $?
	if [ ! -f xine-lib.built ]; then
		echo "  xine-lib wasn't built, ${PROJECT} disabled"
		return -1
	fi

	cd "${TESTSUITE_DOWNLOADDIR}/${MODULE}" || return $?
	[ -x ./configure ] || ./autogen.sh || return $?

#  # pthready jako external library,
#  # use inttypes.h from build-xine-lib if missing
#  ${make} -f Makefile.common all OS_OBJS=setenv.o CFLAGS="${DEBUG_CFLAGS}${CFLAGS} -W -Wall -O2 -I/mingw32/pthreads-win32/include" LIBS="${LDFLAGS} -luser32 -lgdi32 -lcomdlg32 -lcomctl32 -lwinmm -L/mingw32/pthreads-win32/lib -lpthread" XINE_CONFIG=${XINE_LIB_PREFIX}/bin/xine-config HOST=${CROSS}- || return $?
#
#  mv xineui ${DESTDIR}${PREFIX}/bin/xineui.exe || return $?
#  [ -z "${DEBUG}" ] && ${CROSS}-strip ${DESTDIR}${PREFIX}/bin/xineui.exe

	XINE_CFLAGS=$(PKG_CONFIG_PATH="${XINE_LIB_PREFIX}/lib/pkgconfig" pkg-config libxine --cflags --define-variable=prefix="${XINE_LIB_PREFIX}")
	XINE_LIBS=$(PKG_CONFIG_PATH="${XINE_LIB_PREFIX}/lib/pkgconfig" pkg-config libxine --libs --define-variable=prefix="${XINE_LIB_PREFIX}")
	echo "XINE_CFLAGS: $XINE_CFLAGS"
	echo "XINE_LIBS: $XINE_LIBS"

	[ -d "${BUILDDIR}" ] || mkdir -p "${BUILDDIR}" || return $?
	cd "${BUILDDIR}" || return $?
	PKG_CONFIG_PATH="${XINE_LIB_PREFIX}/lib/pkgconfig" ${TESTSUITE_DOWNLOADDIR}/${MODULE}/configure \
		--build=i686-fedora-linux \
		--host=${CROSS} \
		--prefix="${PREFIX}" || return $?
	${make} install \
		DESTDIR=${DESTDIR} \
		XINE_CFLAGS="${XINE_CFLAGS}" \
		XINE_LIBS="${XINE_LIBS}" || return $?
	[ -z "${DEBUG}" ] && ${CROSS}-strip ${DESTDIR}${PREFIX}/bin/xineui.exe

	cd ${TESTSUITE_WORKDIR}

	return 0
}


build_toxine() {
	local install_target
	local all_target

	# resolve different build types: for testsuite and for distribution
	if [ -n "${DESTDIR}" ]; then
		XINE_LIB_PREFIX="${TESTSUITE_DISTDIR}${PREFIX}"
		XINE_LIB_REAL_PREFIX="${PREFIX}"
	else
		XINE_LIB_PREFIX="${PREFIX}"
		XINE_LIB_REAL_PREFIX=$(echo "${PREFIX}" | sed "s,^${TESTSUITE_DISTDIR},,")
	fi

	install_target="install"; [ -z "${DEBUG}" ] || install_target="install-debug"
	all_target="all"; [ -z "${DEBUG}" ] || all_target="all-debug"

	cd ${TESTSUITE_WORKDIR} || return $?
	if [ ! -f xine-lib.built ]; then
		echo "  xine-lib wasn't built, ${PROJECT} disabled"
		return -1
	fi
	cd "${TESTSUITE_DOWNLOADDIR}/${MODULE}" || return $?
	[ -x ./configure ] || ./autogen.sh noconfig || return $?
	[ -d "${BUILDDIR}" ] || mkdir -p "${BUILDDIR}" || return $?
	cd ${BUILDDIR} || return $?

	XINE_CFLAGS=$(${XINE_LIB_PREFIX}/bin/xine-config --cflags)
	XINE_LIBS=$(${XINE_LIB_PREFIX}/bin/xine-config --libs)
	if [ -n "${XINE_LIB_REAL_PREFIX}" ]; then
		XINE_CFLAGS=$(echo ${XINE_CFLAGS} | sed "s,${XINE_LIB_REAL_PREFIX},${XINE_LIB_PREFIX},")
		XINE_LIBS=$(echo ${XINE_LIBS} | sed "s,${XINE_LIB_REAL_PREFIX},${XINE_LIB_PREFIX},")
	fi
#	# TODO: nejaka chybka...
#	echo "  PREFIX=${PREFIX}"
#	echo "  DESTDIR=${DESTDIR}"
#	echo "  XINE_LIB_REAL_PREFIX=${XINE_LIB_REAL_PREFIX}"
#	echo "  XINE_LIB_REAL_PREFIX=${XINE_LIB_REAL_PREFIX}"
#	echo "  XINE_CFLAGS=${XINE_CFLAGS}"
#	echo "  XINE_LIBS=${XINE_LIBS}"

	case "${CROSS}" in
	i386-mingw32|i386-mingw32msvc)
#		PATH="${XINE_LIB_PREFIX}/bin:$PATH" \
#		PKG_CONFIG_PATH="${XINE_LIB_PREFIX}/lib/pkgconfig" \
#		XINE_CFLAGS="-I${XINE_LIB_PREFIX}/include" \
#		XINE_LIBS="-L${XINE_LIB_PREFIX}/lib -lxine -L${MINGW_ZLIB_PREFIX}/lib -lz -L/mingw32/pthreads-win32/lib -lpthread" \
		XINE_CFLAGS="${XINE_CFLAGS}" \
		XINE_LIBS="${XINE_LIBS}" \
		CPPFLAGS="$CPPFLAGS -I/mingw32/include -I/mingw32/pthreads-win32/include" \
		LIBS="$LIBS -L/mingw32/lib -L/mingw32/pthreads-win32/lib" \
		${TESTSUITE_DOWNLOADDIR}/${MODULE}/configure \
			--build=i686-fedora-linux \
			--host=${CROSS} \
			--without-x \
			--without-aalib \
			--with-readline="${MINGW_READLINE_PREFIX}" \
			--disable-xinetest \
			--prefix="${PREFIX}" || return $?

		${make} ${install_target} DESTDIR=${DESTDIR} || return $?

		[ -f "${DESTDIR}${PREFIX}/bin/toxine" ] && mv "${DESTDIR}${PREFIX}/bin/toxine" "${DESTDIR}${PREFIX}/bin/toxine.exe"
		rm -f ${DESTDIR}${PREFIX}/lib/toxine/plugins/*.a
		[ -z "${DEBUG}" ] && ${CROSS}-strip ${DESTDIR}${PREFIX}/{bin/toxine.exe,lib/toxine/plugins/*.dll}

		cp ${MINGW_READLINE_PREFIX}/bin/{history5,readline5,msvcp60}.dll ${DESTDIR}${PREFIX}/bin/ || return $?
		;;
	*-linux-uclibc*|*-uclibc-linux*)
#		PKG_CONFIG_PATH="${XINE_LIB_PREFIX}/lib/pkgconfig" \
#		PATH="${XINE_LIB_PREFIX}/bin:$PATH" \
		XINE_CFLAGS="${XINE_CFLAGS}" \
		XINE_LIBS="${XINE_LIBS}" \
		${TESTSUITE_DOWNLOADDIR}/${MODULE}/configure \
			--build=i686-fedora-linux \
			--host=${CROSS} \
			--without-aalib \
			--with-readline="${UCLIBC_READLINE_PREFIX}" \
			--disable-xinetest \
			--prefix="${PREFIX}" || return $?

		${make} ${all_target} DESTDIR=${DESTDIR} \
			READLINE_LIBS="-L${UCLIBC_READLINE_PREFIX}/lib -lreadline -ltermcap" \
			|| return $?
		${make} ${install_target} DESTDIR=${DESTDIR} \
			READLINE_LIBS="-L${UCLIBC_READLINE_PREFIX}/lib -lreadline -ltermcap" \
			|| return $?
		rm -f ${DESTDIR}${PREFIX}/lib/toxine/plugins/*.a

		;;
	*)
		echo "XINE_CFLAGS=\"${XINE_CFLAGS}\" XINE_LIBS=\"${XINE_LIBS}\" ${TESTSUITE_DOWNLOADDIR}/${MODULE}/configure --disable-xinetest --prefix=\"${PREFIX}\""
		XINE_CFLAGS="${XINE_CFLAGS}" \
		XINE_LIBS="${XINE_LIBS}" \
		${TESTSUITE_DOWNLOADDIR}/${MODULE}/configure \
			--disable-xinetest \
			--prefix="${PREFIX}" || return $?
		${make} ${install_target} DESTDIR=${DESTDIR} || return $?
#		# dirty hack to obey hardcoded plugin dir
#		tree_move "${DESTDIR}${TESTSUITE_DISTDIR}${PREFIX}" "${DESTDIR}${PREFIX}"
#		[ -n "${DESTDIR}" -a x"${DESTDIR}" != x"/" ] && rm -Rf "${DESTDIR}/tmp"
	esac

	cd ${TESTSUITE_WORKDIR}
	return 0
}


function build_xineui {
	local debug

	XINE_LIB_PREFIX="${TESTSUITE_DISTDIR}${PREFIX}"
	# TODO: change to better 'make debug'
	debug=""; [ -z "${DEBUG}" ] || debug="--enable-debug"

	cd "${TESTSUITE_WORKDIR}" || return $?
	if [ ! -f xine-lib.built ]; then
		echo "  xine-lib wasn't built, ${PROJECT} disabled"
		return -1
	fi
	cd "${TESTSUITE_DOWNLOADDIR}/${MODULE}" || return $?
	[ -x ./configure ] || ./autogen.sh noconfig || return $?

	[ -d "${BUILDDIR}" ] || mkdir -p "${BUILDDIR}" || return $?
	cd "${BUILDDIR}" || return $?
	PKG_CONFIG_PATH="${XINE_LIB_PREFIX}/lib/pkgconfig" \
	${TESTSUITE_DOWNLOADDIR}/${MODULE}/configure \
		--with-xine-prefix="${XINE_LIB_PREFIX}" \
		--prefix="${PREFIX}" ${debug} || return $?
	${make} install DESTDIR=${DESTDIR} || return $?
	cd ${TESTSUITE_WORKDIR}

	return 0
}


function check_module() {
	cd "${TESTSUITE_DOWNLOADDIR}/${MODULE}" || return $?
	[ -x ./configure ] || ./autogen.sh noconfig || return $?

	[ -d "$BUILDDIR" ] || mkdir -p "$BUILDDIR" || return $?
	cd "${BUILDDIR}" || return $?
	${TESTSUITE_DOWNLOADDIR}/${MODULE}/configure --prefix=${PREFIX} ${CONFIGURE_OPTS} || return $?
	${make} distcheck DESTDIR=${DESTDIR} || return $?

	cd ${TESTSUITE_WORKDIR}

	return 0
}


#
# create distribution
#
#
function package_project() {
	LIST=""
	PROJECT="$1"
	VERSION="$2"
	TYPE="$3"

	local BASENAME="${PROJECT}-${VERSION}-${PLATFORM_ID}"
	local PACKAGENAME="${BASENAME}.${TYPE}"

# XXX: dirty hack: delete downloads on last testsuite run
	set -o xtrace
	if [ -n "${DELETE_DOWNLOADS}" ]; then
		rm -f ${TESTSUITE_DOWNLOADDIR}/*.tar.gz
	fi

	cd ${TESTSUITE_WORKDIR} || return $?
	mv all.list "${PROJECT}-all.list"
	mv success.list "${PROJECT}-success.list"
	DISTDIR="${TESTSUITE_WORKDIR}/dist/${BASENAME}"
	tree_move "${TESTSUITE_WORKDIR}/dest" "${DISTDIR}" || return $?

	echo "  packing ${PACKAGENAME}"
	rm -f "${PACKAGENAME}"

	case "${TYPE}" in
	tar.gz)
	(cd ${DISTDIR}/..;
	 tar -cf - ${BASENAME} | gzip \
	 	>> "${TESTSUITE_WORKDIR}/${PACKAGENAME}" || return $?) || return $?
	;;

	tar.bz2)
	(cd ${DISTDIR}/..;
	 tar -cf - ${BASENAME} | bzip2 \
		 >> "${TESTSUITE_WORKDIR}/${PACKAGENAME}" || return $?) || return $?
	;;

	zip)
	(cd ${DISTDIR}/..;
	 zip -r "${TESTSUITE_WORKDIR}/${PACKAGENAME}" \
	 	"${BASENAME}" || return $?) || return $?
	;;

	*)
	echo "Unknown type '${TYPE}'"
	return -1
	;;
	esac

	if [ x"${PROJECT}" != x"toxine" ]; then
		tree_move "${DISTDIR}" "${TESTSUITE_DISTDIR}" || return $?
	fi
	rm -Rf "${DISTDIR}"
	# TODO: check this, maybe chrpath will be needed!
	find "${TESTSUITE_DISTDIR}" -name '*.la' -exec rm {} \;

	if cmp "${PROJECT}-all.list" "${PROJECT}-success.list" >/dev/null 2>&1; then
		echo "VERSION=${VERSION}" > "${PROJECT}.info"
		echo "TYPE=${TYPE}" >> "${PROJECT}.info"
		aklog
		upload "${PACKAGENAME}"
		link "${PACKAGENAME}" "${PROJECT}-latest.${TYPE}"
		return 0
	else
		rm -f "${PACKAGENAME}"
		sed -e 's/ /\n/g' "${PROJECT}-all.list" > 1
		sed -e 's/ /\n/g' "${PROJECT}-success.list" > 2
		failed=$(echo $(diff 1 2 | grep '<') | cut -c 3-)
		rm -f 1 2
		echo "${BASENAME} not not packaged, ${failed} not built"
		return -1
	fi
}


#
# post processing on built projects
#
function testsuite_postupload {
	SUITE_DIR="$1"
	shift

	local BASENAME
	local PACKAGENAME

	cd "${TESTSUITE_WORKDIR}"
	while [ -n "$1" ]; do
		PROJECT="$1"
		shift
		if [ ! -f "${PROJECT}.info" ]; then
			echo "  ${PROJECT} not build, no upload"
#FIXME: continue
			return -1;
		fi
		. ${PROJECT}.info

		BASENAME="${PROJECT}-${VERSION}-${PLATFORM_ID}"
		PACKAGENAME="${BASENAME}.${TYPE}"
		md5_file="${PROJECT}-${PLATFORM_ID}.md5"
		cp "${BINSDIR}/${md5_file}" ${md5_file} 2>/dev/null 2>/dev/null || touch ${md5_file}
		if [ `cat ${md5_file} | wc -l` -gt ${MAX_NUMBER} ]; then
			removes=`cat ${md5_file} | head -$(($(cat ${md5_file} | wc -l) - ${MAX_NUMBER})) | awk -F' ' '{print $2};'`
			for file in $removes; do
				remove "$file"
			done
			mv ${md5_file} ${md5_file}.old
			tail -${MAX_NUMBER} ${md5_file}.old > ${md5_file}
			rm -f ${md5_file}.old
		fi
		${md5sum} "${PACKAGENAME}" >> ${md5_file}

		# backup (copying in upload())
		if [ -z "$NO_UPLOAD" ]; then
			MD5FILE="${BINSDIR}/${md5_file}"
			mv "${MD5FILE}.2" "${MD5FILE}.3" 2>/dev/null
			mv "${MD5FILE}.1" "${MD5FILE}.2" 2>/dev/null
			mv "${MD5FILE}.0" "${MD5FILE}.1" 2>/dev/null
			mv "${MD5FILE}" "${MD5FILE}.0"
		fi
		upload ${md5_file}
	done

	cat *.md5 > MD5SUM
	# backup (copying in upload())
	if [ -z "$NO_UPLOAD" ]; then
		MD5FILE="${BINSDIR}/MD5SUM"
		mv "${MD5FILE}.2" "${MD5FILE}.3" 2>/dev/null
		mv "${MD5FILE}.1" "${MD5FILE}.2" 2>/dev/null
		mv "${MD5FILE}.0" "${MD5FILE}.1" 2>/dev/null
		mv "${MD5FILE}" "${MD5FILE}.0"
	fi
	upload MD5SUM

	echo "put MD5SUM" >> testfile.up
	echo "pwd" >> testfile.up

	# finalize upload
	echo "cd /win32/" > testfile
	echo "pwd" >> testfile
	[ -f testfile.rm ] && cat testfile.rm >> testfile
	[ -f testfile.up ] && cat testfile.up >> testfile
	echo "quit" >> testfile
#	[ -n "${NO_UPLOAD}" ] || safetp-ftp-example.pl

	return 0
}


#
# upload the file to all places
#
# requires:
#   $NO_UPLOAD if necessary
#
function upload() {
	local WHAT=$1

	echo "put ${WHAT}" >> testfile.up
	echo "pwd" >> testfile.up

	if [ -z "${NO_UPLOAD}" ]; then
		cp "${WHAT}" ${BINSDIR}
		if [ -n "${AFSDIR_UPLOAD}" ]; then
			cp "${WHAT}" "${AFSDIR_UPLOAD}"
		fi
	fi
}


function link() {
	local WHAT="$1"
	local LINKNAME="$2"
	
	if [ -z "${NO_UPLOAD}" ]; then
		rm -f "${BINSDIR}/${LINKNAME}"
		ln -s  "${WHAT}" "${BINSDIR}/${LINKNAME}"
		if [ -n "${AFSDIR_UPLOAD}" ]; then
			rm -f "${AFSDIR_UPLOAD}/${LINKNAME}"
			ln -s  "${WHAT}" "${AFSDIR_UPLOAD}/${LINKNAME}"
		fi
	fi
}


#
# remove the file from all places
#
# requires:
#   $NO_UPLOAD if necessary
#
function remove() {
	local WHAT=$1

#	echo "rm -f ${WHAT}" >> ${TMPDIR}/testfile.rm
#	echo "pwd" >> ${TMPDIR}/testfile.rm

	if [ -z "${NO_UPLOAD}" ]; then
		rm -f "${BINSDIR}/${WHAT}"
		if [ -n "${AFSDIR_UPLOAD}" ]; then
			rm -f "${AFSDIR_UPLOAD}/${WHAT}" 
		fi
	fi
}


function tree_copy {
	[ -z "$1" -o -z "$2" ] && return -1;
	[ -d "$1" ] || return -1;
	[ -d "$2" ] || mkdir -p "$2" || return $?
	(cd "$1" || return $?
	 tar -cf - * | tar -xf - -C "$2" || return $?
	 cd ..
	) || return $?

	return 0
}


function tree_move {
	[ -z "$1" -o -z "$2" ] && return -1;
	[ -d "$1" ] || return -1;
	[ -d "$2" ] || mkdir -p "$2" || return $?
	(cd "$1" || return $?
	 tar -cf - * | tar -xf - -C "$2" || return $?
	 cd ..
	 rm -rf "$1" || return $?
	) || return $?

	return 0
}


main $@
