dnl
dnl XINE_CHECK_LIB(PKGLIB, LIB, MIN_VERSION, HEADERS, SYMBOL, ACTION-YES, ACTION-NO)
dnl
dnl Searchs LIB library - check the required version, header file and symbol.
dnl Adds --with-package-prefix and --disable-package configure options.
dnl
dnl Defines NAME_LIBS, NAME_CPPFLAGS, HAVE_NAME.
dnl Uses NAME_LIBS, NAME_CPPFLAGS, DEFAULT_NAME_LIBS, DEFAULT_NAME_CPPFLAGS.
dnl
AC_DEFUN([XINE_CHECK_LIB], [
	m4_pushdef([NAME], m4_translit([$1], [a-z.-], [A-Z__]))
	m4_pushdef([name], m4_translit([$1], [A-Z.-], [a-z__]))
	m4_pushdef([option], m4_translit([$1], [A-Z.], [a-z-]))
	_XINE_CHECK_LIB([$1], name, NAME, option, [$2], [$3], [$4], [$5], [$6], [$7])
	m4_popdef([NAME])
	m4_popdef([name])
	m4_popdef([option])
])


dnl
dnl _XINE_CHECK_LIB(PKGLIB, DOWN, UP, OPTION, LIB, MIN_VERSION, HEADERS, SYMBOL, ACTION-YES, ACTION-NO)
dnl
AC_DEFUN([_XINE_CHECK_LIB], [
	dnl
	dnl configure options
	dnl
	AC_ARG_WITH($4-prefix,
	            AC_HELP_STRING([--with-$4-prefix=DIR],
	                           [prefix where $1 is installed]),
	            [$2_handled=yes
	             $3_CPPFLAGS="$DEFAULT_$3_CPPFLAGS $$3_CPPFLAGS -I$withval/include"
	             $3_LIBS="$DEFAULT_$3_LIBS $$3_LIBS -L$withval/$XINE_LIBNAME -l$5"])

	AC_ARG_ENABLE($4,
	              AC_HELP_STRING([--disable-$4],
	                             [do not build $1 support]),
	              with_$2=$enableval, with_$2=yes)
	if test x"$with_$2" = "xno"; then
		$2_handled=yes
	fi

	dnl
	dnl pkg-config
	dnl
	if test x"$$2_handled" != "xyes"; then
		PKG_CHECK_MODULES($3, $1 ifelse([$6], , , >= $6), $2_handled=yes, $2_handled=no)
		$3_CPPFLAGS=$$3_CFLAGS
		$3_CFLAGS=
	fi

	dnl
	dnl $1-config
	dnl
	if test x"$$2_handled" != "xyes"; then
		if test x${$3_CONFIG+set} = x; then
			AC_PATH_TOOL($3_CONFIG, $1-config, no)
		fi
		if test x"$$3_CONFIG" != "xno"; then
			if test -z "$6"; then
				$$2_handled=yes
			else
				$2_version=$($$3_CONFIG --version)
				XINE_CHECK_LIB_CHECK_VERSION([$6], [$$2_version], [$2_handled=yes],
				                            [AC_MSG_RESULT([$1: $6 required, $$2_version found])
				                             with_$2=no])
			fi
			if test x"$$2_handled" = "xyes"; then
				if $$3_CONFIG --cppflags >/dev/null 2>&1; then
					$3_CPPFLAGS="$($$3_CPPFLAGS $$3_CONFIG --cppflags)"
				else
					$3_CPPFLAGS=$($$3_CPPFLAGS $$3_CONFIG --cflags)
				fi
				if $$3_CONFIG --ldflags >/dev/null 2>&1; then
					$3_LIBS="$$3_LIBS $($$3_CONFIG --ldflags)"
				fi
				$3_LIBS="$$3_LIBS $($$3_CONFIG --libs)"
			fi
		fi
	fi

	dnl
	dnl compilation checks
	dnl
	if test x"$with_$2" = "xyes"; then
		ac_save_LIBS=$LIBS
		ac_save_CPPFLAGS=$CPPFLAGS
		CPPFLAGS="$ac_save_CPPFLAGS $$3_CPPFLAGS"
		LIBS="$ac_save_LIBS $$3_LIBS"
		AC_CHECK_HEADERS($7,
		                 AC_CHECK_LIB($5, $8, have_$2=yes,
		                              [$3_CPPFLAGS=""
		                               $3_LIBS=""]))
		CPPFLAGS=$ac_save_CPPFLAGS
		LIBS=$ac_save_LIBS
	fi

	dnl
	dnl results
	dnl
	if test x"$with_$2" = "xyes"; then
		if test x"$have_$2" = "xyes"; then
			ifelse([$9], , :, [$9])
		else
			if test x"$$2_handled" = "xyes"; then
				AC_MSG_WARN([*** Error checking $1 $6. ***
*********************************************************************
Package $1 $6 configuration found but is not properly installed.

Package is searched this way:
1. using prefix from --with-$4-prefix option,
   optionaly with variables DEFAULT_$3_CPPFLAGS and
   DEFAULT_$3_LIBS
2. by pkg-config
3. by $1-config script in path
*********************************************************************])
			else
				ifelse([$10], , :, [$10])
			fi
		fi
	else
		ifelse([$10], , :, [$10])
	fi

	AC_SUBST($3_CPPFLAGS)
	AC_SUBST($3_LIBS)
])


dnl
dnl XINE_CHECK_LIB_CHECK_VERSION(v1, v2, action v1 <= v2, action v1 > v2)
dnl
AC_DEFUN([XINE_CHECK_LIB_CHECK_VERSION], [
	v1_major=$(echo $1 | sed -e 's/\([[0-9]]*\).\([[0-9]]*\).\(.*\)/\1/')
	v1_minor=$(echo $1 | sed -e 's/\([[0-9]]*\).\([[0-9]]*\).\(.*\)/\2/')
	v1_micro=$(echo $1 | sed -e 's/\([[0-9]]*\).\([[0-9]]*\).\(.*\)/\3/')
	v2_major=$(echo $2 | sed -e 's/\([[0-9]]*\).\([[0-9]]*\).\(.*\)/\1/')
	v2_minor=$(echo $2 | sed -e 's/\([[0-9]]*\).\([[0-9]]*\).\(.*\)/\2/')
	v2_micro=$(echo $2 | sed -e 's/\([[0-9]]*\).\([[0-9]]*\).\(.*\)/\3/')

	if test \
		"$v1_major" -lt "$v2_major" -o \
		"$v1_major" -eq "$v2_major" -a \
		"$v1_minor" -lt "$v2_minor" -o \
		"$v1_major" -eq "$v2_major" -a \
		"$v1_minor" -eq "$v2_minor" -a \
		"$v1_micro" -le "$v2_micro" ; then
		ifelse([$3], , :, [$3])
	else
		ifelse([$4], , :, [$4])
	fi
])
